#!/bin/bash

# First, we run the A variant of the server with serial number 1001 for 3 seconds.
# During this time, let the client connect and retrieve a key.
# The timeout command will send a SIGTERM then, tearing down the server.
# We then repeat the experiment with the B variant of the server and
# compare the output.  It better be identical.

test_dir=`dirname ${BASH_SOURCE[0]}`
test_name="identical_keys_from_A_and_B_servers"

# valgrind helpers (active if ${RUN_TESTS_UNDER_VALGRIND}=1)
. ${test_dir}/../../.valgrind

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 B

# The actual test starts here.
timeout 8s ${memcheck1} ./server_example 1001 A &
sleep 2
${memcheck2} ./client_example | grep Data | tail -1 > ${test_name}.A 2>&1
sleep 8
timeout 8s ${memcheck3} ./server_example 1001 B &
sleep 2
${memcheck4} ./client_example | grep Data | tail -1 > ${test_name}.B 2>&1
sleep 8

# NOP unless ${RUN_TESTS_UNDER_VALGRIND}=1
print_valgrind_error_summary

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 B

#Compare test output with reference log.
diff ${test_name}.A ${test_name}.B && diff ${test_name}.A ${test_dir}/reflog
