#!/bin/bash

# We run a server configured to have a low key rate.
# To make matters worse, we request a key almost right away.
#
# The result: the server will let us know that, at the
# moment, there is insufficient key material available.
#
# A longer wait time and/or a higher key rate would change
# the picture, of course.

test_dir=`dirname ${BASH_SOURCE[0]}`
test_name="key_rate_too_low"

# valgrind helpers (active if ${RUN_TESTS_UNDER_VALGRIND}=1)
. ${test_dir}/../../.valgrind

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# The actual test starts here.
timeout 8s ${memcheck1} ./server_example 1001 A ${test_dir}/low_key_rate.cfg &
# Give the server 2s to start up and listen on his port.
sleep 2
${memcheck2} ./client_example ${test_dir}/empty.cfg 0 0 | grep Status > ${test_name}.A 2>&1
sleep 8

# NOP unless ${RUN_TESTS_UNDER_VALGRIND}=1
print_valgrind_error_summary

# Remove lingering semaphores, if any.
env QKD_SERVER_FORCE_SEM_UNLINK=1 ./server_example 1001 A

# Compare test output with reference log.
diff ${test_name}.A ${test_dir}/reflog
