#ifndef ETSI_GS_QKD_004_SERIALIZERS_H
#define ETSI_GS_QKD_004_SERIALIZERS_H

#include "models.h"
#include "qkd/ssl_socket.h"

#ifdef __cplusplus__
extern "C" {
#endif

typedef unsigned char *uuid_tp;

/**
 * The "stream" we are using for ser(ialization)/des(erialization) of data.
 * The one reason we maintain an end pointer is to implement pre- and
 * post-condition assertions in the ser/des routines.
 * - Pre-condition:  pos <  end
 * - Post-condition: pos <= end
 */
typedef struct qkd_stream {
    unsigned char *pos; // pointer to current position
    unsigned char *end; // end marker; one past the last byte of the stream
} qkd_stream_t;

void qkd_uint32_to_stream(uint32_t number, qkd_stream_t *stream);

uint32_t qkd_uint32_from_stream(qkd_stream_t *stream);

void qkd_uuid_to_stream(uuid_t id, qkd_stream_t *stream);

void qkd_uuid_from_stream(uuid_tp id, qkd_stream_t *stream);

void qkd_bytes_data_to_stream(qkd_bytes_t data, qkd_stream_t *stream);

qkd_bytes_t qkd_bytes_data_from_stream(qkd_stream_t *stream);

void qkd_str_to_stream(char *str, qkd_stream_t *stream);

char *qkd_str_from_stream(qkd_stream_t *stream);

void qkd_uri_to_stream(qkd_uri_t uri, qkd_stream_t *stream);

qkd_uri_t qkd_uri_from_stream(qkd_stream_t *stream);

void qkd_mimetype_to_stream(char *mime, qkd_stream_t *stream);

qkd_mimetype_t qkd_mimetype_from_stream(qkd_stream_t *stream);

void qkd_qos_to_stream(qkd_qos_t qos, qkd_stream_t *stream);

qkd_qos_t qkd_qos_from_stream(qkd_stream_t *stream);

void qkd_key_buffer_to_stream(qkd_key_buffer_t data, qkd_stream_t *stream);

qkd_key_buffer_t qkd_key_buffer_from_stream(qkd_stream_t *stream);

void qkd_metadata_to_stream(qkd_key_buffer_t data, qkd_stream_t *stream);

qkd_key_buffer_t qkd_metadata_from_stream(qkd_stream_t *stream);

void qkd_open_connect_request_to_stream(
    qkd_open_connect_request_t open_connect_request, qkd_stream_t *stream);

qkd_open_connect_request_t
qkd_open_connect_request_from_stream(qkd_stream_t *stream);

void qkd_open_connect_response_to_stream(
    qkd_open_connect_response_t open_connect_response, qkd_stream_t *stream);

qkd_open_connect_response_t
qkd_open_connect_response_from_stream(qkd_stream_t *stream);

void qkd_get_key_request_to_stream(qkd_get_key_request_t get_key_request,
                                   qkd_stream_t *stream);

qkd_get_key_request_t qkd_get_key_request_from_stream(qkd_stream_t *stream);

void qkd_get_key_response_to_stream(qkd_get_key_response_t get_key_response,
                                    qkd_stream_t *stream);

qkd_get_key_response_t qkd_get_key_response_from_stream(qkd_stream_t *stream);

void qkd_close_request_to_stream(qkd_close_request_t close_request,
                                 qkd_stream_t *stream);

qkd_close_request_t qkd_close_request_from_stream(qkd_stream_t *stream);

void qkd_close_response_to_stream(qkd_close_response_t close_response,
                                  qkd_stream_t *stream);

qkd_close_response_t qkd_close_response_from_stream(qkd_stream_t *stream);

void qkd_header_to_stream(qkd_header_t header, qkd_stream_t *stream);

qkd_header_t qkd_header_from_stream(qkd_stream_t *stream);

#ifdef __cplusplus__
}
#endif

#endif // ETSI_GS_QKD_004_SERIALIZERS_H
