#include <assert.h>
#include <malloc.h>

#include "qkd/copy.h"
#include "qkd/sizes.h"

void qkd_copy_str(char **dst, char **src) {
    assert(src != NULL);
    size_t str_size = qkd_sizeof_str(*src);
    free(*dst);
    *dst = malloc(str_size);
    if (*dst == NULL) {
        fprintf(stderr, "ERROR: out of memory\n");
        exit(qkd_status_code_out_of_memory);
    }
    memcpy(*dst, *src, str_size);
}

void qkd_copy_uri(qkd_uri_t *dst, qkd_uri_t *src) {
    assert(src != NULL);
    assert(dst != NULL);
    qkd_copy_str(dst, src);
}

void qkd_copy_mimetype(qkd_mimetype_t *dst, qkd_mimetype_t *src) {
    assert(src != NULL);
    assert(dst != NULL);
    qkd_copy_str(dst, src);
}

void qkd_copy_qos(qkd_qos_t *dst, qkd_qos_t *src) {
    assert(src != NULL);
    assert(dst != NULL);
    dst->key_chunk_size = src->key_chunk_size;
    dst->max_bps = src->max_bps;
    dst->min_bps = src->min_bps;
    dst->capacity = src->capacity;
    dst->timeout = src->timeout;
    dst->ttf = src->ttf;
    qkd_copy_mimetype(&dst->oc_metadata_mimetype, &src->oc_metadata_mimetype);
    qkd_copy_mimetype(&dst->gk_metadata_mimetype, &src->gk_metadata_mimetype);
}
