# Tests

The `tests` directory contains automated tests.


## Top-level Test Driver

We use CTest as a test driver.

See the toplevel README.md file for instructions how to run tests.


## Test Options

### Valgrind

In order to run tests under [valgrind](https://valgrind.org/), set
${RUN_TESTS_UNDER_VALGRIND} to 1.  Tests then produce valgrind log
files (`valgrind.log.<PID>.<counter>`) in the current directory.

If you run tests directly, i.e., not via CTest, then an error summary
is displayed.  For instance
``` shell
cd build_dir
../examples/identical_keys_from_A_and_B_servers/run_test.sh
```
should generate something along the lines of
``` shell
Unlinking semaphore and exiting
Unlinking semaphore and exiting
[INFO] Running server on "127.0.0.1:25575"...
<removed some lines here>
==407426== ERROR SUMMARY: 0 errors from 0 contexts (suppressed: 4 from 1)
==407470== ERROR SUMMARY: 0 errors from 0 contexts (suppressed: 4 from 1)
==407603== ERROR SUMMARY: 0 errors from 0 contexts (suppressed: 4 from 1)
==407635== ERROR SUMMARY: 0 errors from 0 contexts (suppressed: 4 from 1)
<removed some more lines here>
```

Valgrind is configured to use a suppressions file
(`etsi-gs-qkd-004-c/valgrind.supp`) that addresses a known OpenSSL
issue.

If you run tests via CTest, e.g.
``` shell
ctest -R short
```
to run all tests containing `short` in their names then you can
generate a valgrind error summary via
``` shell
grep "ERROR SUMMARY" valgrind.log.*
```

### Static Code Checks

Static code analysis (via `cppcheck`) is run automatically when the
code is built.  See below for more details.

### Code Coverage

TO DO: Add support for automated code coverage checks.


## Types of Tests

Currently, we maintain three basic types of tests.

1) Automatic execution of examples from the `examples` directory.
2) Additional unit/system tests meant to increase test coverage of
   reachable lines of code.
3) Static checks via cppcheck (automatically carried out as part of
   the build process).


## Test Coverage Metrics

In the future, we may look at more elaborate test coverage metrics.

For now, we only measure and observe the following:

1) Functional test coverage as implicitly defined by the set of
   examples.
2) TO DO: Code line coverage.

Test coverage metrics we may consider in the future include (but are
not limited to)

* more-complete functional coverage,
* condition/expression coverage,
* FSM coverage including
    - state coverage as expressed by
        * enum values and/or
        * value ranges of certain variables and
    - transition/toggle coverage.

Also, we do not perform fuzz testing (at the network protocol level)
yet.


## Test Coverage Goals

We seek to eventually achieve 100% coverage of reachable lines of
code.  The initial release aimed at first public review of the ETSI GS
QKD 004 reference implementation may not reach this goal though.

In terms of functional coverage, we aim at the following set of
examples:

* QKD device pair yielding matching keys
* `GET_KEY` with NULL and non-NULL KSID
* `GET_KEY` with NULL and non-NULL index
* Finite key rate and capacity
    - Keys cannot be consumed arbitrarily fast; key server responds
      with the correct (error) status code when client keeps calling
      `GET_KEY` when key supply is (temporarily) exhausted
    - Key server responds with correct (error) status if an index is
      too big (i.e., "too far away" from the indices of the keys
      currently held in the key manager's finite-size key storage)
* Unused keys are deleted in time due to TTF expiry
* Trying to access pre-consumed key (a.k.a. "double dipping attempt")
  leads to a proper (error) response from the key server
* Simple QoS negotiation (key server can/cannot fulfill min-rate
  or capacity requirements)

In the future, we may want to add tests that also cover:

* Unknown destination KME and/or SAE ID
    - Key server does not know target key server associated with the
      target URI (in a multi-hop scenario)
    - Key server cannot determine (multi-hop) route to target key
      server (e.g., due to connection failures or due to lack of
      available bandwidth)
